Developer Documentation

> **Note**
>
> This document is still in development and may be partially out of
> date.

In general, a hook function is one that the Apache HTTP Server will call
at some point during the processing of a request. Modules can provide
functions that are called, and specify when they get called in
comparison to other modules.

Core Hooks {#corehooks}
==========

The httpd\'s core modules offer a predefinined list of hooks used during
the standard

request processing

phase. Creating a new hook will expose a function that implements it
(see sections below) but it is essential to understand that you will not
extend the httpd\'s core hooks. Their presence and order in the request
processing is in fact a consequence of how they are called in

server/request.c

(check

this section

for an overview). The core hooks are listed in the

doxygen documentation

.

Reading

guide for developing modules

and

request processing

before proceeding is highly recommended.

Creating a hook function {#create}
========================

In order to create a new hook, four things need to be done:

Declare the hook function {#create-declare}
-------------------------

Use the

AP\_DECLARE\_HOOK

macro, which needs to be given the return type of the hook function, the
name of the hook, and the arguments. For example, if the hook returns an

int

and takes a

request\_rec \*

and an

int

and is called

do\_something

, then declare it like this:

AP\_DECLARE\_HOOK(int, do\_something, (request\_rec \*r, int n))

This should go in a header which modules will include if they want to
use the hook.

Create the hook structure {#create-create}
-------------------------

Each source file that exports a hook has a private structure which is
used to record the module functions that use the hook. This is declared
as follows:

APR\_HOOK\_STRUCT( APR\_HOOK\_LINK(do\_something) \... )

Implement the hook caller {#create-implement}
-------------------------

The source file that exports the hook has to implement a function that
will call the hook. There are currently three possible ways to do this.
In all cases, the calling function is called

ap\_run\_

hookname

()

.

### Void hooks

If the return value of a hook is

void

, then all the hooks are called, and the caller is implemented like
this:

AP\_IMPLEMENT\_HOOK\_VOID(do\_something, (request\_rec \*r, int n), (r,
n))

The second and third arguments are the dummy argument declaration and
the dummy arguments as they will be used when calling the hook. In other
words, this macro expands to something like this:

void ap\_run\_do\_something(request\_rec \*r, int n) { \...
do\_something(r, n); }

### Hooks that return a value

If the hook returns a value, then it can either be run until the first
hook that does something interesting, like so:

AP\_IMPLEMENT\_HOOK\_RUN\_FIRST(int, do\_something, (request\_rec \*r,
int n), (r, n), DECLINED)

The first hook that does

not

return

DECLINED

stops the loop and its return value is returned from the hook caller.
Note that

DECLINED

is the traditional hook return value meaning \"I didn\'t do anything\",
but it can be whatever suits you.

Alternatively, all hooks can be run until an error occurs. This boils
down to permitting

two

return values, one of which means \"I did something, and it was OK\" and
the other meaning \"I did nothing\". The first function that returns a
value other than one of those two stops the loop, and its return is the
return value. Declare these like so:

AP\_IMPLEMENT\_HOOK\_RUN\_ALL(int, do\_something, (request\_rec \*r, int
n), (r, n), OK, DECLINED)

Again,

OK

and

DECLINED

are the traditional values. You can use what you want.

Call the hook callers {#create-call}
---------------------

At appropriate moments in the code, call the hook caller, like so:

int n, ret; request\_rec \*r; ret=ap\_run\_do\_something(r, n);

Hooking the hook {#hooking}
================

A module that wants a hook to be called needs to do two things.

Implement the hook function {#hooking-implement}
---------------------------

Include the appropriate header, and define a static function of the
correct type:

static int my\_something\_doer(request\_rec \*r, int n) { \... return
OK; }

Add a hook registering function {#hooking-add}
-------------------------------

During initialisation, the server will call each modules hook
registering function, which is included in the module structure:

static void my\_register\_hooks() {
ap\_hook\_do\_something(my\_something\_doer, NULL, NULL,
APR\_HOOK\_MIDDLE); } mode MODULE\_VAR\_EXPORT my\_module = { \...
my\_register\_hooks /\* register hooks \*/ };

Controlling hook calling order {#hooking-order}
------------------------------

In the example above, we didn\'t use the three arguments in the hook
registration function that control calling order of all the functions
registered within the hook. There are two mechanisms for doing this. The
first, rather crude, method, allows us to specify roughly where the hook
is run relative to other modules. The final argument control this. There
are three possible values:

APR\_HOOK\_FIRST

,

APR\_HOOK\_MIDDLE

and

APR\_HOOK\_LAST

.

All modules using any particular value may be run in any order relative
to each other, but, of course, all modules using

APR\_HOOK\_FIRST

will be run before

APR\_HOOK\_MIDDLE

which are before

APR\_HOOK\_LAST

. Modules that don\'t care when they are run should use

APR\_HOOK\_MIDDLE

.

These values are spaced out, so that positions like

APR\_HOOK\_FIRST-2

are possible to hook slightly earlier than other functions.

Note that there are two more values,

APR\_HOOK\_REALLY\_FIRST

and

APR\_HOOK\_REALLY\_LAST

. These should only be used by the hook exporter.

The other method allows finer control. When a module knows that it must
be run before (or after) some other modules, it can specify them by
name. The second (third) argument is a NULL-terminated array of strings
consisting of the names of modules that must be run before (after) the
current module. For example, suppose we want \"mod\_xyz.c\" and
\"mod\_abc.c\" to run before we do, then we\'d hook as follows:

static void register\_hooks() { static const char \* const aszPre\[\] =
{ \"mod\_xyz.c\", \"mod\_abc.c\", NULL };
ap\_hook\_do\_something(my\_something\_doer, aszPre, NULL,
APR\_HOOK\_MIDDLE); }

Note that the sort used to achieve this is stable, so ordering set by

APR\_HOOK\_

ORDER

is preserved, as far as is possible.
