Developer Documentation

This is a first attempt at writing the lessons I learned when trying to
convert the

mod\_mmap\_static

module to Apache 2.0. It\'s by no means definitive and probably won\'t
even be correct in some ways, but it\'s a start.

The easier changes \... {#easy}
=======================

Cleanup Routines {#cleanup}
----------------

These now need to be of type

apr\_status\_t

and return a value of that type. Normally the return value will be

APR\_SUCCESS

unless there is some need to signal an error in the cleanup. Be aware
that even though you signal an error not all code yet checks and acts
upon the error.

Initialisation Routines {#init}
-----------------------

These should now be renamed to better signify where they sit in the
overall process. So the name gets a small change from

mmap\_init

to

mmap\_post\_config

. The arguments passed have undergone a radical change and now look like

apr\_pool\_t \*p

apr\_pool\_t \*plog

apr\_pool\_t \*ptemp

server\_rec \*s

Data Types {#datatypes}
----------

A lot of the data types have been moved into the

APR

. This means that some have had a name change, such as the one shown
above. The following is a brief list of some of the changes that you are
likely to have to make.

pool

becomes

apr\_pool\_t

table

becomes

apr\_table\_t

The messier changes\... {#messy}
=======================

Register Hooks
--------------

The new architecture uses a series of hooks to provide for calling your
functions. These you\'ll need to add to your module by way of a new
function,

static void register\_hooks(void)

. The function is really reasonably straightforward once you understand
what needs to be done. Each function that needs calling at some stage in
the processing of a request needs to be registered, handlers do not.
There are a number of phases where functions can be added, and for each
you can specify with a high degree of control the relative order that
the function will be called in.

This is the code that was added to

mod\_mmap\_static

:

static void register\_hooks(void) { static const char \* const
aszPre\[\]={ \"http\_core.c\",NULL };
ap\_hook\_post\_config(mmap\_post\_config,NULL,NULL,HOOK\_MIDDLE);
ap\_hook\_translate\_name(mmap\_static\_xlat,aszPre,NULL,HOOK\_LAST); };

This registers 2 functions that need to be called, one in the

post\_config

stage (virtually every module will need this one) and one for the

translate\_name

phase. note that while there are different function names the format of
each is identical. So what is the format?

ap\_hook\_

phase\_name

(

function\_name

,

predecessors

,

successors

,

position

);

There are 3 hook positions defined\...

HOOK\_FIRST

HOOK\_MIDDLE

HOOK\_LAST

To define the position you use the position and then modify it with the
predecessors and successors. Each of the modifiers can be a list of
functions that should be called, either before the function is run
(predecessors) or after the function has run (successors).

In the

mod\_mmap\_static

case I didn\'t care about the

post\_config

stage, but the

mmap\_static\_xlat

must

be called after the core module had done its name translation, hence the
use of the aszPre to define a modifier to the position

HOOK\_LAST

.

Module Definition {#moddef}
-----------------

There are now a lot fewer stages to worry about when creating your
module definition. The old definition looked like

module MODULE\_VAR\_EXPORT

module\_name

\_module = { STANDARD\_MODULE\_STUFF, /\* initializer \*/ /\* dir config
creater \*/ /\* dir merger \-\-- default is to override \*/ /\* server
config \*/ /\* merge server config \*/ /\* command handlers \*/ /\*
handlers \*/ /\* filename translation \*/ /\* check\_user\_id \*/ /\*
check auth \*/ /\* check access \*/ /\* type\_checker \*/ /\* fixups \*/
/\* logger \*/ /\* header parser \*/ /\* child\_init \*/ /\* child\_exit
\*/ /\* post read-request \*/ };

The new structure is a great deal simpler\...

module MODULE\_VAR\_EXPORT

module\_name

\_module = { STANDARD20\_MODULE\_STUFF, /\* create per-directory config
structures \*/ /\* merge per-directory config structures \*/ /\* create
per-server config structures \*/ /\* merge per-server config structures
\*/ /\* command handlers \*/ /\* handlers \*/ /\* register hooks \*/ };

Some of these read directly across, some don\'t. I\'ll try to summarise
what should be done below.

The stages that read directly across :

/\* dir config creater \*/

/\* create per-directory config structures \*/

/\* server config \*/

/\* create per-server config structures \*/

/\* dir merger \*/

/\* merge per-directory config structures \*/

/\* merge server config \*/

/\* merge per-server config structures \*/

/\* command table \*/

/\* command apr\_table\_t \*/

/\* handlers \*/

/\* handlers \*/

The remainder of the old functions should be registered as hooks. There
are the following hook stages defined so far\...

ap\_hook\_pre\_config

do any setup required prior to processing configuration directives

ap\_hook\_check\_config

review configuration directive interdependencies

ap\_hook\_test\_config

executes only with

-t

option

ap\_hook\_open\_logs

open any specified logs

ap\_hook\_post\_config

this is where the old

\_init

routines get registered

ap\_hook\_http\_method

retrieve the http method from a request. (legacy)

ap\_hook\_auth\_checker

check if the resource requires authorization

ap\_hook\_access\_checker

check for module-specific restrictions

ap\_hook\_check\_user\_id

check the user-id and password

ap\_hook\_default\_port

retrieve the default port for the server

ap\_hook\_pre\_connection

do any setup required just before processing, but after accepting

ap\_hook\_process\_connection

run the correct protocol

ap\_hook\_child\_init

call as soon as the child is started

ap\_hook\_create\_request

??

ap\_hook\_fixups

last chance to modify things before generating content

ap\_hook\_handler

generate the content

ap\_hook\_header\_parser

lets modules look at the headers, not used by most modules, because they
use

post\_read\_request

for this

ap\_hook\_insert\_filter

to insert filters into the filter chain

ap\_hook\_log\_transaction

log information about the request

ap\_hook\_optional\_fn\_retrieve

retrieve any functions registered as optional

ap\_hook\_post\_read\_request

called after reading the request, before any other phase

ap\_hook\_quick\_handler

called before any request processing, used by cache modules.

ap\_hook\_translate\_name

translate the URI into a filename

ap\_hook\_type\_checker

determine and/or set the doc type
