mod\_auth\_digest

User authentication using MD5 Digest Authentication

Extension

mod\_auth\_digest.c

auth\_digest\_module

This module implements HTTP Digest Authentication (

RFC2617

), and provides an alternative to

mod\_auth\_basic

where the password is not transmitted as cleartext. However, this does

not

lead to a significant security advantage over basic authentication. On
the other hand, the password storage on the server is much less secure
with digest authentication than with basic authentication. Therefore,
using basic auth and encrypting the whole connection using

mod\_ssl

is a much better alternative.

AuthName

AuthType

Require

Authentication howto

Using Digest Authentication {#using}
===========================

To use MD5 Digest authentication, configure the location to be protected
as shown in the below example:

\<Location \"/private/\"\> AuthType Digest AuthName \"private area\"
AuthDigestDomain \"/private/\" \"http://mirror.my.dom/private2/\"
AuthDigestProvider file AuthUserFile \"/web/auth/.digest\_pw\" Require
valid-user \</Location\>

AuthDigestDomain

should list the locations that will be protected by this configuration.

The password file referenced in the

AuthUserFile

directive may be created and managed using the

htdigest

tool.

> **Note**
>
> Digest authentication was intended to be more secure than basic
> authentication, but no longer fulfills that design goal. A
> man-in-the-middle attacker can trivially force the browser to
> downgrade to basic authentication. And even a passive eavesdropper can
> brute-force the password using today\'s graphics hardware, because the
> hashing algorithm used by digest authentication is too fast. Another
> problem is that the storage of the passwords on the server is
> insecure. The contents of a stolen htdigest file can be used directly
> for digest authentication. Therefore using
>
> mod\_ssl
>
> to encrypt the whole connection is strongly recommended.
>
> mod\_auth\_digest
>
> only works properly on platforms where APR supports shared memory.

AuthDigestProvider

Sets the authentication provider(s) for this location

AuthDigestProvider

provider-name

\[

provider-name

\] \...

AuthDigestProvider file

directory

.htaccess

AuthConfig

The

AuthDigestProvider

directive sets which provider is used to authenticate the users for this
location. The default

file

provider is implemented by the

mod\_authn\_file

module. Make sure that the chosen provider module is present in the
server.

See

mod\_authn\_dbm

,

mod\_authn\_file

,

mod\_authn\_dbd

and

mod\_authn\_socache

for providers.

AuthDigestQop

Determines the quality-of-protection to use in digest authentication

AuthDigestQop none\|auth\|auth-int \[auth\|auth-int\]

AuthDigestQop auth

directory

.htaccess

AuthConfig

The

AuthDigestQop

directive determines the

quality-of-protection

to use.

auth

will only do authentication (username/password);

auth-int

is authentication plus integrity checking (an MD5 hash of the entity is
also computed and checked);

none

will cause the module to use the old RFC-2069 digest algorithm (which
does not include integrity checking). Both

auth

and

auth-int

may be specified, in which the case the browser will choose which of
these to use.

none

should only be used if the browser for some reason does not like the
challenge it receives otherwise.

> **Note**
>
> auth-int
>
> is not implemented yet.

AuthDigestNonceLifetime

How long the server nonce is valid

AuthDigestNonceLifetime

seconds

AuthDigestNonceLifetime 300

directory

.htaccess

AuthConfig

The

AuthDigestNonceLifetime

directive controls how long the server nonce is valid. When the client
contacts the server using an expired nonce the server will send back a
401 with

stale=true

. If

seconds

is greater than 0 then it specifies the amount of time for which the
nonce is valid; this should probably never be set to less than 10
seconds. If

seconds

is less than 0 then the nonce never expires.

AuthDigestNonceFormat

Determines how the nonce is generated

AuthDigestNonceFormat

format

directory

.htaccess

AuthConfig

> **Note**
>
> Not implemented yet.

AuthDigestNcCheck

Enables or disables checking of the nonce-count sent by the server

AuthDigestNcCheck On\|Off

AuthDigestNcCheck Off

server config

> **Note**
>
> Not implemented yet.

AuthDigestAlgorithm

Selects the algorithm used to calculate the challenge and response
hashes in digest authentication

AuthDigestAlgorithm MD5\|MD5-sess

AuthDigestAlgorithm MD5

directory

.htaccess

AuthConfig

The

AuthDigestAlgorithm

directive selects the algorithm used to calculate the challenge and
response hashes.

> **Note**
>
> MD5-sess
>
> is not correctly implemented yet.

AuthDigestDomain

URIs that are in the same protection space for digest authentication

AuthDigestDomain

URI

\[

URI

\] \...

directory

.htaccess

AuthConfig

The

AuthDigestDomain

directive allows you to specify one or more URIs which are in the same
protection space (

i.e.

use the same realm and username/password info). The specified URIs are
prefixes; the client will assume that all URIs \"below\" these are also
protected by the same username/password. The URIs may be either absolute
URIs (

i.e.

including a scheme, host, port, etc.) or relative URIs.

This directive

should

always be specified and contain at least the (set of) root URI(s) for
this space. Omitting to do so will cause the client to send the
Authorization header for

every request

sent to this server. Apart from increasing the size of the request, it
may also have a detrimental effect on performance if

AuthDigestNcCheck

is on.

The URIs specified can also point to different servers, in which case
clients (which understand this) will then share username/password info
across multiple servers without prompting the user each time.

AuthDigestShmemSize

The amount of shared memory to allocate for keeping track of clients

AuthDigestShmemSize

size

AuthDigestShmemSize 1000

server config

The

AuthDigestShmemSize

directive defines the amount of shared memory, that will be allocated at
the server startup for keeping track of clients. Note that the shared
memory segment cannot be set less than the space that is necessary for
tracking at least

one

client. This value is dependent on your system. If you want to find out
the exact value, you may simply set

AuthDigestShmemSize

to the value of

0

and read the error message after trying to start the server.

The

size

is normally expressed in Bytes, but you may follow the number with a

K

or an

M

to express your value as KBytes or MBytes. For example, the following
directives are all equivalent:

AuthDigestShmemSize 1048576 AuthDigestShmemSize 1024K
AuthDigestShmemSize 1M
