mpm\_common

A collection of directives that are implemented by more than one
multi-processing module (MPM)

MPM

CoreDumpDirectory

Directory where Apache HTTP Server attempts to switch before dumping
core

CoreDumpDirectory

directory

See usage for the default setting

server config

event

worker

prefork

This controls the directory to which Apache httpd attempts to switch
before dumping core. If your operating system is configured to create
core files in the working directory of the crashing process,

CoreDumpDirectory

is necessary to change working directory from the default

ServerRoot

directory, which should not be writable by the user the server runs as.

If you want a core dump for debugging, you can use this directive to
place it in a different location. This directive has no effect if your
operating system is not configured to write core files to the working
directory of the crashing processes.

> **Note**
>
> Using this directive on Linux may allow other processes on the system
> (if running with similar privileges, such as CGI scripts) to attach to
> httpd children via the
>
> ptrace
>
> system call. This may make weaken the protection from certain security
> attacks. It is not recommended to use this directive on production
> systems.

> **Note**
>
> If Apache httpd starts as root and switches to another user, the Linux
> kernel
>
> disables
>
> core dumps even if the directory is writable for the process. Apache
> httpd (2.0.46 and later) reenables core dumps on Linux 2.4 and beyond,
> but only if you explicitly configure a
>
> CoreDumpDirectory
>
> .

> **Note**
>
> To enable core-dumping of suid-executables on BSD-systems (such as
> FreeBSD), set
>
> kern.sugid\_coredump
>
> to 1.

> **Note**
>
> CoreDumpDirectory
>
> processing only occurs for a select set of fatal signals: SIGFPE,
> SIGILL, SIGABORT, SIGSEGV, and SIGBUS.
>
> On some operating systems, SIGQUIT also results in a core dump but
> does not go through
>
> CoreDumpDirectory
>
> or
>
> EnableExceptionHook
>
> processing, so the core location is dictated entirely by the operating
> system.

EnableExceptionHook

Enables a hook that runs exception handlers after a crash

EnableExceptionHook On\|Off

EnableExceptionHook Off

server config

event

worker

prefork

For safety reasons this directive is only available if the server was
configured with the

\--enable-exception-hook

option. It enables a hook that allows external modules to plug in and do
something after a child crashed.

There are already two modules,

mod\_whatkilledus

and

mod\_backtrace

that make use of this hook. Please have a look at Jeff Trawick\'s

EnableExceptionHook site

for more information about these.

GracefulShutdownTimeout

Specify a timeout after which a gracefully shutdown server will exit.

GracefulShutdownTimeout

seconds

GracefulShutdownTimeout 0

server config

event

worker

prefork

The

GracefulShutdownTimeout

specifies how many seconds after receiving a \"graceful-stop\" signal, a
server should continue to run, handling the existing connections.

Setting this value to zero means that the server will wait indefinitely
until all remaining requests have been fully served.

PidFile

File where the server records the process ID of the daemon

PidFile

filename

PidFile httpd.pid

server config

event

worker

prefork

mpm\_winnt

mpmt\_os2

The

PidFile

directive sets the file to which the server records the process id of
the daemon. If the filename is not absolute, then it is assumed to be
relative to the

DefaultRuntimeDir

.

PidFile /var/run/apache.pid

It is often useful to be able to send the server a signal, so that it
closes and then re-opens its

ErrorLog

and

TransferLog

, and re-reads its configuration files. This is done by sending a SIGHUP
(kill -1) signal to the process id listed in the

PidFile

.

The

PidFile

is subject to the same warnings about log file placement and

security

.

> **Note**
>
> As of Apache HTTP Server 2, we recommended that you only use the
>
> apachectl
>
> script, or the init script that your OS provides, for (re-)starting or
> stopping the server.

Listen

IP addresses and ports that the server listens to

Listen \[

IP-address

:\]

portnumber

\[

protocol

\] \[options=

flag

\[,

flag..

\]\]

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

mpmt\_os2

The optional

options=

argument is available in httpd 2.5.1 and later.

The

Listen

directive instructs Apache httpd to listen to only specific IP addresses
or ports; by default it responds to requests on all IP interfaces.

Listen

is now a required directive. If it is not in the config file, the server
will fail to start. This is a change from previous versions of Apache
httpd.

The

Listen

directive tells the server to accept incoming requests on the specified
port or address-and-port combination. If only a port number is
specified, the server listens to the given port on all interfaces. If an
IP address is given as well as a port, the server will listen on the
given port and interface.

Multiple

Listen

directives may be used to specify a number of addresses and ports to
listen to. The server will respond to requests from any of the listed
addresses and ports.

For example, to make the server accept connections on both port 80 and
port 8000, use:

Listen 80 Listen 8000

To make the server accept connections on two specified interfaces and
port numbers, use

Listen 192.170.2.1:80 Listen 192.170.2.5:8000

IPv6 addresses must be surrounded in square brackets, as in the
following example:

Listen \[2001:db8::a00:20ff:fea7:ccea\]:80

In httpd 2.5.1 and later, if built against APR 1.7.0 or later, scoped
(or \"zoned\") literal IPv6 addresses are accepted in the

Listen

directive, such as the following example.

Listen \[fe80::4bfe:88aa:5d42:64d0%eth2\]:8081

The optional

protocol

argument is not required for most configurations. If not specified,

https

is the default for port 443 and

http

the default for all other ports. The protocol is used to determine which
module should handle a request, and to apply protocol specific
optimizations with the

AcceptFilter

directive.

You only need to set the protocol if you are running on non-standard
ports. For example, running an

https

site on port 8443:

Listen 192.170.2.1:8443 https

The optional

options=flag,flag\...

argument can be used to enable certain socket options for the listening
port. These options are not required for most configurations and should
be used with care. Availability of each flag varies across operating
systems. The available

flag

s are:

freebind

: The

IP\_FREEBIND

socket option is enabled, allowing a Listen directive to be used for an
address which is not (yet) available on the system. (Linux only)

reuseport

: The

SO\_REUSEPORT

socket option is enabled, allowing a Listen directive to bind to a port
which may already be in use by another process.

v6only

: The

IPV6\_V6ONLY

socket option is enabled, allowing a Listen directive to bind to an IPv6
address without also accepting connections via IPv4, or conflicting with
a Listen directive using an IPv4 address bound to the same port. (If the
server is built with IPv4-mapped addresses

disabled

, this is the default behaviour and this option has no effect.)

> **Note**
>
> Multiple
>
> Listen
>
> directives for the same IP address and port will result in an
>
> Address already in use
>
> error message.

DNS Issues

Setting which addresses and ports Apache HTTP Server uses

Further discussion of the

Address already in use

error message, including other causes.

ListenCoresBucketsRatio

Ratio between the number of CPU cores (online) and the number of
listeners\' buckets

ListenCoresBucketsRatio

ratio

ListenCoresBucketsRatio 0 (disabled)

server config

event

worker

prefork

Available in Apache HTTP Server 2.4.17, with a kernel supporting the
socket option

SO\_REUSEPORT

and distributing new connections evenly across listening processes\' (or
threads\') sockets using it (eg. Linux 3.9 and later, but not the
current implementations of

SO\_REUSEPORT

in \*BSDs.

A

ratio

between the number of (online) CPU cores and the number of listeners\'
buckets can be used to make Apache HTTP Server create

num\_cpu\_cores / ratio

listening buckets, each containing its own

Listen

-ing socket(s) on the same port(s), and then make each child handle a
single bucket (with round-robin distribution of the buckets at children
creation time).

> **Note**
>
> On Linux (and also BSD) a CPU core can be turned on/off if
>
> Hotplug
>
> is configured, therefore
>
> ListenCoresBucketsRatio
>
> needs to take this parameter into account while calculating the number
> of buckets to create.

ListenCoresBucketsRatio

can improve the scalability when accepting new connections is/becomes
the bottleneck. On systems with a large number of CPU cores, enabling
this feature has been tested to show significant performances
improvement and shorter responses time.

There must be at least twice the number of CPU cores than the configured

ratio

for this to be active. The recommended

ratio

is

8

, hence at least

16

cores should be available at runtime when this value is used. The right

ratio

to obtain maximum performance needs to be calculated for each target
system, testing multiple values and observing the variations in your key
performance metrics.

This directive influences the calculation of the

MinSpareThreads

and

MaxSpareThreads

lower bound values. The number of children processes needs to be a
multiple of the number of buckets to optimally accept connections.

> **Note**
>
> Setting the
>
> SO\_REUSEPORT
>
> option on the listening socket(s) consequently allows multiple
> processes (sharing the same
>
> EUID
>
> , e.g.
>
> root
>
> ) to bind to the the same IP address and port, without the binding
> error raised by the system in the usual case.
>
> This also means that multiple instances of Apache httpd configured on
> a same
>
> IP:port
>
> and with a positive
>
> ListenCoresBucketsRatio
>
> would start without an error too, and then run with incoming
> connections evenly distributed across both instances (this is NOT a
> recommendation or a sensible usage in any case, but just a notice that
> it would prevent such possible issues to be detected).
>
> Within the same instance, Apache httpd will check and fail to start if
> multiple
>
> Listen
>
> directives on the exact same IP (or hostname) and port are configured,
> thus avoiding the creation of some duplicated buckets which would be
> useless and kill performances. However it can\'t (and won\'t try
> harder to) catch all the possible overlapping cases (like a hostname
> resolving to an IP used elsewhere).

ListenBackLog

Maximum length of the queue of pending connections

ListenBackLog

backlog

ListenBackLog 511

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

mpmt\_os2

The maximum length of the queue of pending connections. Generally no
tuning is needed or desired; however, on some systems, it is desirable
to increase this when under a TCP SYN flood attack. See the backlog
parameter to the

listen(2)

system call.

This will often be limited to a smaller number by the operating system.
This varies from OS to OS. Also note that many OSes do not use exactly
what is specified as the backlog, but use a number based on (but
normally larger than) what is set.

MaxRequestWorkers

Maximum number of connections that will be processed simultaneously

MaxRequestWorkers

number

See usage for details

server config

event

worker

prefork

The

MaxRequestWorkers

directive sets the limit on the number of simultaneous requests that
will be served. Any connection attempts over the

MaxRequestWorkers

limit will normally be queued, up to a number based on the

ListenBacklog

directive. Once a child process is freed at the end of a different
request, the connection will then be serviced.

For non-threaded servers (

i.e.

,

prefork

),

MaxRequestWorkers

translates into the maximum number of child processes that will be
launched to serve requests. The default value is

256

; to increase it, you must also raise

ServerLimit

.

For threaded and hybrid servers (

e.g.

event

or

worker

),

MaxRequestWorkers

restricts the total number of threads that will be available to serve
clients. For hybrid MPMs, the default value is

16

(

ServerLimit

) multiplied by the value of

25

(

ThreadsPerChild

). Therefore, to increase

MaxRequestWorkers

to a value that requires more than 16 processes, you must also raise

ServerLimit

.

MaxRequestWorkers

was called

MaxClients

before version 2.3.13. The old name is still supported.

MaxMemFree

Maximum amount of memory that the main allocator is allowed to hold
without calling

free()

MaxMemFree

KBytes

MaxMemFree 2048

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

The

MaxMemFree

directive sets the maximum number of free Kbytes that every allocator is
allowed to hold without calling

free()

. In threaded MPMs, every thread has its own allocator. When set to
zero, the threshold will be set to unlimited.

MaxConnectionsPerChild

Limit on the number of connections that an individual child server will
handle during its life

MaxConnectionsPerChild

number

MaxConnectionsPerChild 0

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

mpmt\_os2

Available Apache HTTP Server 2.3.9 and later. The old name

MaxRequestsPerChild

is still supported.

The

MaxConnectionsPerChild

directive sets the limit on the number of connections that an individual
child server process will handle. After

MaxConnectionsPerChild

connections, the child process will die. If

MaxConnectionsPerChild

is

0

, then the process will never expire.

Setting

MaxConnectionsPerChild

to a non-zero value limits the amount of memory that a process can
consume by (accidental) memory leakage.

MaxSpareThreads

Maximum number of idle threads

MaxSpareThreads

number

See usage for details

server config

event

worker

mpm\_netware

mpmt\_os2

Maximum number of idle threads. Different MPMs deal with this directive
differently.

For

worker

and

event

, the default is

MaxSpareThreads 250

. These MPMs deal with idle threads on a server-wide basis. If there are
too many idle threads in the server, then child processes are killed
until the number of idle threads is less than this number. Additional
processes/threads might be created if

ListenCoresBucketsRatio

is enabled.

For

mpm\_netware

the default is

MaxSpareThreads 100

. Since this MPM runs a single-process, the spare thread count is also
server-wide.

mpmt\_os2

works similar to

mpm\_netware

. For

mpmt\_os2

the default value is

10

.

> **Note**
>
> The range of the
>
> MaxSpareThreads
>
> value is restricted. Apache httpd will correct the given value
> automatically according to the following rules:
>
> mpm\_netware
>
> wants the value to be greater than
>
> MinSpareThreads
>
> .
>
> For
>
> worker
>
> and
>
> event
>
> , the value must be greater or equal to the sum of
>
> MinSpareThreads
>
> and
>
> ThreadsPerChild
>
> .

MinSpareThreads

StartServers

MaxSpareServers

MinSpareThreads

Minimum number of idle threads available to handle request spikes

MinSpareThreads

number

See usage for details

server config

event

worker

mpm\_netware

mpmt\_os2

Minimum number of idle threads to handle request spikes. Different MPMs
deal with this directive differently.

worker

and

event

use a default of

MinSpareThreads 75

and deal with idle threads on a server-wide basis. If there aren\'t
enough idle threads in the server, then child processes are created
until the number of idle threads is greater than

number

. Additional processes/threads might be created if

ListenCoresBucketsRatio

is enabled.

mpm\_netware

uses a default of

MinSpareThreads 10

and, since it is a single-process MPM, tracks this on a server-wide
basis.

mpmt\_os2

works similar to

mpm\_netware

. For

mpmt\_os2

the default value is

5

.

MaxSpareThreads

StartServers

MinSpareServers

ScoreBoardFile

Location of the file used to store coordination data for the child
processes

ScoreBoardFile

file-path

ScoreBoardFile apache\_runtime\_status

server config

event

worker

prefork

mpm\_winnt

Apache HTTP Server uses a scoreboard to communicate between its parent
and child processes. Some architectures require a file to facilitate
this communication. If the file is left unspecified, Apache httpd first
attempts to create the scoreboard entirely in memory (using anonymous
shared memory) and, failing that, will attempt to create the file on
disk (using file-based shared memory). Specifying this directive causes
Apache httpd to always create the file on the disk.

If

file-path

is not an absolute path, the location specified will be relative to the
value of

DefaultRuntimeDir

.

ScoreBoardFile /var/run/apache\_runtime\_status

File-based shared memory is useful for third-party applications that
require direct access to the scoreboard.

If you use a

ScoreBoardFile

, then you may see improved speed by placing it on a RAM disk. But be
careful that you heed the same warnings about log file placement and

security

.

Stopping and Restarting Apache HTTP Server

ReceiveBufferSize

TCP receive buffer size

ReceiveBufferSize

bytes

ReceiveBufferSize 0

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

mpmt\_os2

The server will set the TCP receive buffer size to the number of bytes
specified.

If set to the value of

0

, the server will use the OS default.

SendBufferSize

TCP buffer size

SendBufferSize

bytes

SendBufferSize 0

server config

event

worker

prefork

mpm\_winnt

mpm\_netware

mpmt\_os2

Sets the server\'s TCP send buffer size to the number of bytes
specified. It is often useful to set this past the OS\'s standard
default value on high speed, high latency connections (

i.e.

, 100ms or so, such as transcontinental fast pipes).

If set to the value of

0

, the server will use the default value provided by your OS.

Further configuration of your operating system may be required to elicit
better performance on high speed, high latency connections.

> **Note**
>
> On some operating systems, changes in TCP behavior resulting from a
> larger
>
> SendBufferSize
>
> may not be seen unless
>
> EnableSendfile
>
> is set to OFF. This interaction applies only to static files.

ServerLimit

Upper limit on configurable number of processes

ServerLimit

number

See usage for details

server config

event

worker

prefork

For the

prefork

MPM, this directive sets the maximum configured value for

MaxRequestWorkers

for the lifetime of the Apache httpd process. For the

worker

and

event

MPMs, this directive in combination with

ThreadLimit

sets the maximum configured value for

MaxRequestWorkers

for the lifetime of the Apache httpd process. For the

event

MPM, this directive also defines how many old server processes may keep
running and finish processing open connections. Any attempts to change
this directive during a restart will be ignored, but

MaxRequestWorkers

can be modified during a restart.

Special care must be taken when using this directive. If

ServerLimit

is set to a value much higher than necessary, extra, unused shared
memory will be allocated. If both

ServerLimit

and

MaxRequestWorkers

are set to values higher than the system can handle, Apache httpd may
not start or the system may become unstable.

With the

prefork

MPM, use this directive only if you need to set

MaxRequestWorkers

higher than 256 (default). Do not set the value of this directive any
higher than what you might want to set

MaxRequestWorkers

to.

With

worker

, use this directive only if your

MaxRequestWorkers

and

ThreadsPerChild

settings require more than 16 server processes (default). Do not set the
value of this directive any higher than the number of server processes
required by what you may want for

MaxRequestWorkers

and

ThreadsPerChild

.

With

event

, increase this directive if the process number defined by your

MaxRequestWorkers

and

ThreadsPerChild

settings, plus the number of gracefully shutting down processes, is more
than 16 server processes (default).

> **Note**
>
> There is a hard limit of
>
> ServerLimit 20000
>
> compiled into the server (for the
>
> prefork
>
> MPM 200000). This is intended to avoid nasty effects caused by typos.
> To increase it even further past this limit, you will need to modify
> the value of MAX\_SERVER\_LIMIT in the mpm source file and rebuild the
> server.

Stopping and Restarting Apache HTTP Server

StartServers

Number of child server processes created at startup

StartServers

number

See usage for details

server config

event

worker

prefork

mpmt\_os2

The

StartServers

directive sets the number of child server processes created on startup.
As the number of processes is dynamically controlled depending on the
load, (see

MinSpareThreads

,

MaxSpareThreads

,

MinSpareServers

,

MaxSpareServers

) there is usually little reason to adjust this parameter.

The default value differs from MPM to MPM.

worker

and

event

default to

StartServers 3

;

prefork

defaults to

5

;

mpmt\_os2

defaults to

2

.

StartThreads

Number of threads created on startup

StartThreads

number

See usage for details

server config

mpm\_netware

Number of threads created on startup. As the number of threads is
dynamically controlled depending on the load, (see

MinSpareThreads

,

MaxSpareThreads

,

MinSpareServers

,

MaxSpareServers

) there is usually little reason to adjust this parameter.

For

mpm\_netware

the default is

StartThreads 50

and, since there is only a single process, this is the total number of
threads created at startup to serve requests.

ThreadLimit

Sets the upper limit on the configurable number of threads per child
process

ThreadLimit

number

See usage for details

server config

event

worker

mpm\_winnt

This directive sets the maximum configured value for

ThreadsPerChild

for the lifetime of the Apache httpd process. Any attempts to change
this directive during a restart will be ignored, but

ThreadsPerChild

can be modified during a restart up to the value of this directive.

Special care must be taken when using this directive. If

ThreadLimit

is set to a value much higher than

ThreadsPerChild

, extra unused shared memory will be allocated. If both

ThreadLimit

and

ThreadsPerChild

are set to values higher than the system can handle, Apache httpd may
not start or the system may become unstable. Do not set the value of
this directive any higher than your greatest predicted setting of

ThreadsPerChild

for the current run of Apache httpd.

The default value for

ThreadLimit

is

1920

when used with

mpm\_winnt

and

64

when used with the others.

> **Note**
>
> There is a hard limit of
>
> ThreadLimit 20000
>
> (or
>
> ThreadLimit 100000
>
> with
>
> event
>
> ,
>
> ThreadLimit 15000
>
> with
>
> mpm\_winnt
>
> ) compiled into the server. This is intended to avoid nasty effects
> caused by typos. To increase it even further past this limit, you will
> need to modify the value of MAX\_THREAD\_LIMIT in the mpm source file
> and rebuild the server.

ThreadsPerChild

Number of threads created by each child process

ThreadsPerChild

number

See usage for details

server config

event

worker

mpm\_winnt

This directive sets the number of threads created by each child process.
The child creates these threads at startup and never creates more. If
using an MPM like

mpm\_winnt

, where there is only one child process, this number should be high
enough to handle the entire load of the server. If using an MPM like

worker

, where there are multiple child processes, the

total

number of threads should be high enough to handle the common load on the
server.

The default value for

ThreadsPerChild

is

64

when used with

mpm\_winnt

and

25

when used with the others.

The value of

ThreadsPerChild

can not exceed the value of

ThreadLimit

. If a higher value is configured, it will be automatically reduced at
start-up and a warning will be logged. The relationship between these 2
directives is explained in

ThreadLimit

.

ThreadStackSize

The size in bytes of the stack used by threads handling client
connections

ThreadStackSize

size

65536 on NetWare; varies on other operating systems

server config

event

worker

mpm\_winnt

mpm\_netware

mpmt\_os2

The

ThreadStackSize

directive sets the size of the stack (for autodata) of threads which
handle client connections and call modules to help process those
connections. In most cases the operating system default for stack size
is reasonable, but there are some conditions where it may need to be
adjusted:

On platforms with a relatively small default thread stack size (e.g.,
HP-UX), Apache httpd may crash when using some third-party modules which
use a relatively large amount of autodata storage. Those same modules
may have worked fine on other platforms where the default thread stack
size is larger. This type of crash is resolved by setting

ThreadStackSize

to a value higher than the operating system default. This type of
adjustment is necessary only if the provider of the third-party module
specifies that it is required, or if diagnosis of an Apache httpd crash
indicates that the thread stack size was too small.

On platforms where the default thread stack size is significantly larger
than necessary for the web server configuration, a higher number of
threads per child process will be achievable if

ThreadStackSize

is set to a value lower than the operating system default. This type of
adjustment should only be made in a test environment which allows the
full set of web server processing to be exercised, as there may be
infrequent requests which require more stack to process. The minimum
required stack size strongly depends on the modules used, but any change
in the web server configuration can invalidate the current

ThreadStackSize

setting.

On Linux, this directive can only be used to increase the default stack
size, as the underlying system call uses the value as a

minimum

stack size. The (often large) soft limit for

ulimit -s

(8MB if unlimited) is used as the default stack size.

> **Note**
>
> It is recommended to not reduce
>
> ThreadStackSize
>
> unless a high number of threads per child process is needed. On some
> platforms (including Linux), a setting of 128000 is already too low
> and causes crashes with some common modules.

AcceptErrorsNonFatal

Treat some errors accepting a new connection as non-fatal to the httpd
process.

AcceptErrorsNonFatal ON

OFF (ECONNREFUSED, ECONNABORTED, ECONNRESET cause the process to exit)

server config

event

worker

prefork

2.5.1 and later

The

AcceptErrorsNonFatal

alters the servers behavior under some rare errors that may occur while
accepting a new client connection. By default, the child process
handling a request will gracefully exit when nearly any socket error
occurs during the accept() system call. This is to ensure a potentially
unhealthy child process does not try to take on more new connections.

With

AcceptErrorsNonFatal

set to \"ON\", the process will

not

begin to exit if the accept() error is ECONNREFUSED, ECONNABORTED, or
ECONNRESET.

> **Note**
>
> Some third-party firwewall software components may inject errors into
> accept() processing, using return codes not specified by the operating
> system.
