Rewrite

This document supplements the

mod\_rewrite

reference documentation

. It provides a few advanced techniques using mod\_rewrite.

> **Note**
>
> Note that many of these examples won\'t work unchanged in your
> particular server configuration, so it\'s important that you
> understand them, rather than merely cutting and pasting the examples
> into your configuration.

Module documentation

mod\_rewrite introduction

Redirection and remapping

Controlling access

Virtual hosts

Proxying

Using RewriteMap

When not to use mod\_rewrite

URL-based sharding across multiple backends {#sharding}
===========================================

Description:

A common technique for distributing the burden of server load or storage
space is called \"sharding\". When using this method, a front-end server
will use the url to consistently \"shard\" users or objects to separate
backend servers.

Solution:

A mapping is maintained, from users to target servers, in external map
files. They look like:

user1 physical\_host\_of\_user1

user2 physical\_host\_of\_user2

\# \... and so on

We put this into a

map.users-to-hosts

file. The aim is to map;

/u/user1/anypath

to

http://physical\_host\_of\_user1/u/user/anypath

thus every URL path need not be valid on every backend physical host.
The following ruleset does this for us with the help of the map files
assuming that server0 is a default server which will be used if a user
has no entry in the map:

RewriteEngine on RewriteMap users-to-hosts
\"txt:/path/to/map.users-to-hosts\" RewriteRule
\"\^/u/(\[\^/\]+)/?(.\*)\"
\"http://\${users-to-hosts:\$1\|server0}/u/\$1/\$2\"

See the

RewriteMap

documentation and the

RewriteMap HowTo

for more discussion of the syntax of this directive.

On-the-fly Content-Regeneration {#on-the-fly-content}
===============================

Description:

We wish to dynamically generate content, but store it statically once it
is generated. This rule will check for the existence of the static file,
and if it\'s not there, generate it. The static files can be removed
periodically, if desired (say, via cron) and will be regenerated on
demand.

Solution:

This is done via the following ruleset:

\# This example is valid in per-directory context only RewriteCond
\"%{REQUEST\_URI}\" !-U RewriteRule \"\^(.+)\\.html\$\"
\"/regenerate\_page.cgi\" \[PT,L\]

The

-U

operator determines whether the test string (in this case,

REQUEST\_URI

) is a valid URL. It does this via a subrequest. In the event that this
subrequest fails - that is, the requested resource doesn\'t exist - this
rule invokes the CGI program

/regenerate\_page.cgi

, which generates the requested resource and saves it into the document
directory, so that the next time it is requested, a static copy can be
served.

In this way, documents that are infrequently updated can be served in
static form. if documents need to be refreshed, they can be deleted from
the document directory, and they will then be regenerated the next time
they are requested.

Load Balancing
==============

Description:

We wish to randomly distribute load across several servers using
mod\_rewrite.

Solution:

We\'ll use

RewriteMap

and a list of servers to accomplish this.

RewriteEngine on RewriteMap lb \"rnd:/path/to/serverlist.txt\"
RewriteRule \"\^/(.\*)\" \"http://\${lb:servers}/\$1\" \[P,L\]

serverlist.txt

will contain a list of the servers:

\#\# serverlist.txt

servers one.example.com\|two.example.com\|three.example.com

If you want one particular server to get more of the load than the
others, add it more times to the list.

Discussion

Apache comes with a load-balancing module -

mod\_proxy\_balancer

\- which is far more flexible and featureful than anything you can
cobble together using mod\_rewrite.

Structured Userdirs {#structuredhomedirs}
===================

Description:

Some sites with thousands of users use a structured homedir layout,

i.e.

each homedir is in a subdirectory which begins (for instance) with the
first character of the username. So,

/\~larry/anypath

is

/home/

l

/larry/public\_html/anypath

while

/\~waldo/anypath

is

/home/

w

/waldo/public\_html/anypath

.

Solution:

We use the following ruleset to expand the tilde URLs into the above
layout.

RewriteEngine on RewriteRule \"\^/\~(

(\[a-z\])

\[a-z0-9\]+)(.\*)\" \"/home/

\$2

/\$1/public\_html\$3\"

Redirecting Anchors {#redirectanchors}
===================

Description:

By default, redirecting to an HTML anchor doesn\'t work, because
mod\_rewrite escapes the

\#

character, turning it into

\%23

. This, in turn, breaks the redirection.

Solution:

Use the

\[NE\]

flag on the

RewriteRule

. NE stands for No Escape.

Discussion:

This technique will of course also work with other special characters
that mod\_rewrite, by default, URL-encodes.

Time-Dependent Rewriting {#time-dependent}
========================

Description:

We wish to use mod\_rewrite to serve different content based on the time
of day.

Solution:

There are a lot of variables named

TIME\_xxx

for rewrite conditions. In conjunction with the special lexicographic
comparison patterns

\<STRING

,

\>STRING

and

=STRING

we can do time-dependent redirects:

RewriteEngine on RewriteCond \"%{TIME\_HOUR}%{TIME\_MIN}\" \>0700
RewriteCond \"%{TIME\_HOUR}%{TIME\_MIN}\" \<1900 RewriteRule
\"\^foo\\.html\$\" \"foo.day.html\" \[L\] RewriteRule \"\^foo\\.html\$\"
\"foo.night.html\"

This provides the content of

foo.day.html

under the URL

foo.html

from

07:01-18:59

and at the remaining time the contents of

foo.night.html

.

> **Note**
>
> mod\_cache
>
> , intermediate proxies and browsers may each cache responses and cause
> the either page to be shown outside of the time-window configured.
>
> mod\_expires
>
> may be used to control this effect. You are, of course, much better
> off simply serving the content dynamically, and customizing it based
> on the time of day.

Set Environment Variables Based On URL Parts {#setenvvars}
============================================

Description:

At times, we want to maintain some kind of status when we perform a
rewrite. For example, you want to make a note that you\'ve done that
rewrite, so that you can check later to see if a request came via that
rewrite. One way to do this is by setting an environment variable.

Solution:

Use the \[E\] flag to set an environment variable.

RewriteEngine on RewriteRule \"\^/horse/(.\*)\" \"/pony/\$1\" \[E=

rewritten:1

\]

Later in your ruleset you might check for this environment variable
using a RewriteCond:

RewriteCond \"%{ENV:rewritten}\" =1

Note that environment variables do not survive an external redirect. You
might consider using the \[CO\] flag to set a cookie. For per-directory
and htaccess rewrites, where the final substitution is processed as an
internal redirect, environment variables from the previous round of
rewriting are prefixed with \"REDIRECT\_\".
